(***********************************************
 *                                             *
 *              CS520 Fall 2006                *
 *           Programming Assignment            *
 *         Computer Science Department         *
 *             Boston University               *
 *                                             *
 ***********************************************)

(*
 * The lexical analyzer which converts a character
 * stream to tokens.
 *)

{
open Support.Error

let reservedWords = [
  (* Keywords *)
  ("lam", fun i -> Parser.LAMBDA i);
  
  (* Symbols *)
  ("(", fun i -> Parser.LPAREN i); 
  (")", fun i -> Parser.RPAREN i);

  (* Special compound symbols: *)
  ("=>", fun i -> Parser.DARROW i);
]

(* Support functions *)

type buildfun = info -> Parser.token

let (symbolTable : (string, buildfun) Hashtbl.t) = Hashtbl.create 1024

let _ =
  List.iter (fun (str,f) -> Hashtbl.add symbolTable str f) reservedWords

let createID i str =
  try (Hashtbl.find symbolTable str) i
  with _ -> Parser.VAR {i=i; v=str}

let lineno   = ref 1
and depth    = ref 0
and start    = ref 0
and filename = ref ""
and startLex = ref dummyinfo

let create inFile stream =
  if not (Filename.is_implicit inFile) then filename := inFile
  else filename := Filename.concat (Sys.getcwd()) inFile;
  lineno := 1; start := 0; Lexing.from_channel stream

let newline lexbuf = incr lineno; start := (Lexing.lexeme_start lexbuf)

let info lexbuf =
  createInfo (!filename) (!lineno) (Lexing.lexeme_start lexbuf - !start)

let text = Lexing.lexeme
}

(* The main body of the lexical analyzer *)

rule main = parse
  [' ' '\009' '\012']+     
    { main lexbuf }

| [' ' '\009' '\012']*("\r")?"\n" 
    { newline lexbuf; main lexbuf }

| "*)" 
    { error (info lexbuf) "Unmatched end of comment" }

| "(*" 
    { depth := 1; startLex := info lexbuf; comment lexbuf; main lexbuf }

| ['A'-'Z' 'a'-'z' '_'] ['A'-'Z' 'a'-'z' '_' '0'-'9' '\'']*
    { createID (info lexbuf) (text lexbuf) }

| "=>" 
    { createID (info lexbuf) (text lexbuf) }

| ['(' ')'] 
    { createID (info lexbuf) (text lexbuf) }

| eof 
    { Parser.EOF }

| _  
    { error (info lexbuf) "Illegal character" }

and comment = parse
  "(*"
    { depth := succ !depth; comment lexbuf }
| "*)"
    { depth := pred !depth; if !depth > 0 then comment lexbuf }
| eof
    { error (!startLex) "Comment not terminated" }
| [^ '\n']
    { comment lexbuf }
| "\n"
    { newline lexbuf; comment lexbuf }


